use crate::controllers::HttpResult;
use crate::extractors::auth_extractor::AuthExtractor;
use crate::services::{files_service, movements_service};
use actix_web::HttpResponse;

#[derive(serde::Serialize)]
struct GlobalStats {
    global_balance: f32,
    number_movements: usize,
    number_files: usize,
    total_files_size: u64,
}

/// Get global statistics
pub async fn global(auth: AuthExtractor) -> HttpResult {
    let movements = movements_service::get_all_movements_user(auth.user.id()).await?;
    let files = files_service::get_all_files_user(auth.user.id()).await?;

    Ok(HttpResponse::Ok().json(GlobalStats {
        global_balance: movements.iter().fold(0.0, |sum, m| sum + m.amount),
        number_movements: movements.len(),
        number_files: files.len(),
        total_files_size: files.iter().fold(0, |sum, m| sum + m.file_size as u64),
    }))
}
